import { Switch, Route } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { Toaster } from "@/components/ui/toaster";
import { TooltipProvider } from "@/components/ui/tooltip";
import { SEO } from "@/components/seo";
import { SearchProvider } from "@/hooks/use-search-context";
import { Suspense, lazy, useEffect } from "react";
import { PerformanceOptimizer } from "@/components/performance-optimizer";
import { Loader2 } from "lucide-react";

// Eagerly loaded critical components (fast initial load)
import Intro from "@/pages/intro";
import NotFound from "@/pages/not-found";

// Lazy load all other pages to reduce initial bundle size
const Home = lazy(() => import("@/pages/home"));
const About = lazy(() => import("@/pages/about"));
const Jobs = lazy(() => import("@/pages/jobs"));
const Lab = lazy(() => import("@/pages/lab"));
const WhatIsAI = lazy(() => import("@/pages/what-is-ai"));
const Robotics = lazy(() => import("@/pages/robotics"));
const SubcategoryPage = lazy(() => import("@/pages/subcategory"));
const RoboticsTutorials = lazy(() => import("@/pages/robotics-tutorials"));
const AITutorials = lazy(() => import("@/pages/ai-tutorials"));
const AIHardware = lazy(() => import("@/pages/ai-hardware"));
const PrivacyPolicy = lazy(() => import("@/pages/privacy"));
const AIAgents = lazy(() => import("@/pages/ai-agents"));
const AIFrameworks = lazy(() => import("@/pages/ai-frameworks"));
const Learning = lazy(() => import("@/pages/learning"));
const ModelContextProtocol = lazy(() => import("@/pages/model-context-protocol"));
const AgentToAgentProtocol = lazy(() => import("@/pages/agent-to-agent-protocol"));
const AgentCommunicationProtocol = lazy(() => import("@/pages/agent-communication-protocol"));
const Community = lazy(() => import("@/pages/community"));
const ServerlessAI = lazy(() => import("@/pages/serverless-ai"));
const WebotsTest = lazy(() => import("@/pages/webots-test"));

// Loading fallback component with reduced CLS impact
const PageLoader = () => (
  <div className="fixed inset-0 flex items-center justify-center bg-background bg-opacity-50 backdrop-blur-sm z-50">
    <Loader2 className="h-12 w-12 animate-spin text-primary" aria-label="Loading page..." />
  </div>
);

// Simple component that redirects to another path
const Redirect = ({ to }: { to: string }) => {
  // Set a timeout to give the browser time to render before redirecting
  setTimeout(() => {
    window.location.href = to;
  }, 0);
  
  // Show a loading indicator while the redirect happens
  return <PageLoader />;
};

function Router() {
  
  return (
    <Switch>
      {/* Handle root path - check for search query */}
      <Route path="/" component={() => {
        // Check if there's a search query
        const urlParams = new URLSearchParams(window.location.search);
        const hasSearchQuery = urlParams.has('q');
        
        if (hasSearchQuery) {
          // If there's a search query, go to the home page with the query
          return <Redirect to={`/home${window.location.search}`} />;
        } else {
          // Otherwise redirect to intro page on initial visit
          return <Redirect to="/intro" />;
        }
      }} />
      
      {/* Eagerly loaded routes for critical paths */}
      <Route path="/intro" component={Intro} />
      
      {/* Lazy loaded routes wrapped in Suspense for better performance */}
      <Route path="/home">
        <Suspense fallback={<PageLoader />}>
          <Home />
        </Suspense>
      </Route>
      <Route path="/about">
        <Suspense fallback={<PageLoader />}>
          <About />
        </Suspense>
      </Route>
      <Route path="/jobs">
        <Suspense fallback={<PageLoader />}>
          <Jobs />
        </Suspense>
      </Route>
      <Route path="/lab">
        <Suspense fallback={<PageLoader />}>
          <Lab />
        </Suspense>
      </Route>
      <Route path="/what-is-ai">
        <Suspense fallback={<PageLoader />}>
          <WhatIsAI />
        </Suspense>
      </Route>
      <Route path="/robotics">
        <Suspense fallback={<PageLoader />}>
          <Robotics />
        </Suspense>
      </Route>
      <Route path="/subcategory/:subcategory">
        <Suspense fallback={<PageLoader />}>
          <SubcategoryPage />
        </Suspense>
      </Route>
      <Route path="/category/:category">
        <Suspense fallback={<PageLoader />}>
          <SubcategoryPage />
        </Suspense>
      </Route>
      <Route path="/model-context-protocol">
        <Suspense fallback={<PageLoader />}>
          <ModelContextProtocol />
        </Suspense>
      </Route>
      <Route path="/category/model-context-protocol">
        <Suspense fallback={<PageLoader />}>
          <ModelContextProtocol />
        </Suspense>
      </Route>
      <Route path="/agent-to-agent-protocol">
        <Suspense fallback={<PageLoader />}>
          <AgentToAgentProtocol />
        </Suspense>
      </Route>
      <Route path="/category/agent-to-agent-protocol">
        <Suspense fallback={<PageLoader />}>
          <AgentToAgentProtocol />
        </Suspense>
      </Route>
      <Route path="/agent-communication-protocol">
        <Suspense fallback={<PageLoader />}>
          <AgentCommunicationProtocol />
        </Suspense>
      </Route>
      <Route path="/category/agent-communication-protocol">
        <Suspense fallback={<PageLoader />}>
          <AgentCommunicationProtocol />
        </Suspense>
      </Route>
      <Route path="/robotics-tutorials">
        <Suspense fallback={<PageLoader />}>
          <RoboticsTutorials />
        </Suspense>
      </Route>
      <Route path="/ai-tutorials">
        <Suspense fallback={<PageLoader />}>
          <AITutorials />
        </Suspense>
      </Route>
      <Route path="/ai-hardware">
        <Suspense fallback={<PageLoader />}>
          <AIHardware />
        </Suspense>
      </Route>
      <Route path="/ai-agents">
        <Suspense fallback={<PageLoader />}>
          <AIAgents />
        </Suspense>
      </Route>
      <Route path="/ai-frameworks">
        <Suspense fallback={<PageLoader />}>
          <AIFrameworks />
        </Suspense>
      </Route>
      <Route path="/serverless-ai">
        <Suspense fallback={<PageLoader />}>
          <ServerlessAI />
        </Suspense>
      </Route>
      <Route path="/learning">
        <Suspense fallback={<PageLoader />}>
          <Learning />
        </Suspense>
      </Route>
      <Route path="/privacy">
        <Suspense fallback={<PageLoader />}>
          <PrivacyPolicy />
        </Suspense>
      </Route>
      <Route path="/community">
        <Suspense fallback={<PageLoader />}>
          <Community />
        </Suspense>
      </Route>
      {/* Redirect from /resource/:id to appropriate page with resource parameter */}
      <Route path="/resource/:id" component={() => {
        const id = window.location.pathname.split('/resource/')[1];
        if (!id) return <NotFound />;
        
        const resourceId = parseInt(id);
        if (isNaN(resourceId)) return <NotFound />;
        
        // Redirect based on resource ID ranges
        if (resourceId >= 2000 && resourceId < 3000) {
          // AI Agent resources
          return <Redirect to={`/ai-agents?resource=${resourceId}`} />;
        } else if (resourceId >= 1000 && resourceId < 2000) {
          // Serverless AI resources
          return <Redirect to={`/serverless-ai?resource=${resourceId}`} />;
        } else if (resourceId >= 3000 && resourceId < 4000) {
          // Model context resources
          return <Redirect to={`/model-context-protocol?resource=${resourceId}`} />;
        } else if (resourceId >= 4000 && resourceId < 5000) {
          // AI Hardware resources
          return <Redirect to={`/ai-hardware?resource=${resourceId}`} />;
        } else {
          // Default to home page with resource parameter
          return <Redirect to={`/home?resource=${resourceId}`} />;
        }
      }} />
      {/* Redirect from /webots-info to home page with search parameter */}
      <Route path="/webots-info" component={() => {
        return <Redirect to="/home?q=webots" />;
      }} />
      <Route path="/webots-test">
        <Suspense fallback={<PageLoader />}>
          <WebotsTest />
        </Suspense>
      </Route>
      <Route component={NotFound} />
    </Switch>
  );
}

function App() {
  return (
    <QueryClientProvider client={queryClient}>
      <SearchProvider>
        <TooltipProvider>
          <SEO />
          <Toaster />
          <Router />
        </TooltipProvider>
      </SearchProvider>
    </QueryClientProvider>
  );
}

export default App;